/*
**> ex3.prg
**
** Gourlay, A.R. and S. McKee [1977], The construction of hopscotch methods
** for parabolic and elliptic equations in two space dimensions with a mixed
** derivative, Journal of Computational and Applied Mathematics, 3, 201-205
**
** page 205, example 4
**
** a(t,x,y) = (x - 0.5)^2
** b(t,x,y) = 0.5 * (x - 0.5) * (y - 0.5)
** c(t,x,y) = (y - 0.5)^2
** d(t,x,y) = 0
** e(t,x,y) = 0
** f(t,x,y) = 0
** g(t,x,y) = -(5*x*y*(x+y) + (x+y) - (6*x*y + x^2 + y^2))*exp(-t)
**
** u(0,x,y) = x*y*(x+y)
** u(t,0,y) = 0
** u(t,1,y) = (y + y^2) * exp(-t)
** u(t,x,0) = 0
** u(t,x,1) = (x + x^2) * exp(-t)
**
** h(t,x,y,u) = u
*/

new;
library PDE2D,pgraph;
PDE2Dset;


proc aProc(t,x,y);
  retp( ((x - 0.5)^2) .* ones(1,cols(y)) );
endp;

proc bProc(t,x,y);
  retp(0.5*(x - 0.5).*(y - 0.5));
endp;

proc cProc(t,x,y);
  retp( ((y - 0.5)^2) .* ones(rows(x),1) );
endp;

proc dProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc eProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( -(5*x.*y.*(x+y) + (x+y) - (6*x.*y + x^2 + y^2)).*exp(-t) );
endp;

proc tminBound(t,x,y);
  retp( (x^2).*y + x.*(y^2) );
endp;

proc xminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc xmaxBound(t,x,y);
  retp( (y + y^2) .* exp(-t) );
endp;

proc yminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc ymaxBound(t,x,y);
  retp( (x + x^2) .* exp(-t) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
               &tminBound,&xminBound,&xmaxBound,&yminBound,&ymaxBound,
               0,0,0,0);

xmin = 0; xmax = 1; Nx = 21;
ymin = 0; ymax = 1; Ny = 21;
tmin = 0; tmax = 0.25; Nt = 101;

_Hopscotch_PrintIters = 1;

_Hopscotch_Methods = 1|2;
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ex3");

x = ExtractSolution("ex3","x");
y = ExtractSolution("ex3","y");
t = ExtractSolution("ex3","t");

indx = FindIndex(x,0.25|0.5|0.75);
indy = FindIndex(y',0.5);

U = ExtractSolution("ex3","y"|y[indy]);
U1 = U[indx,.];

_Hopscotch_Methods = 2|3;
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ex3");

x = ExtractSolution("ex3","x");
y = ExtractSolution("ex3","y");
t = ExtractSolution("ex3","t");

indx = FindIndex(x,0.25|0.5|0.75);
indy = FindIndex(y',0.5);

U = ExtractSolution("ex3","y"|y[indy]);
U2 = U[indx,.];


/* The exact solution is 
**
**          u(t,x,y) = (x^2*y + x*y^2) * exp(-t)
**
*/

proc solutionProc(t,x,y);
  retp( ( (x^2).*y + x.*(y^2) ) * exp(-t) );
endp;

sol = solutionProc(t',0.25|0.5|0.75,0.5);

graphset;
  begwind;
  window(1,2,0);
  _pdate = ""; _pnum = 2; _pnumht = 0.25; _paxht = 0.25; _ptitlht = 0.25;
  _plctrl = -3; _pstype = 8|9|10; _pysci = 1; _psymsiz = 7;
  xtics(0,0.25,0.05,5);
  xlabel("t");
  ylabel("Numerical error for y = 0.5");
  _plegstr = "x = 0.25\000x = 0.50\000x = 0.75";

  setwind(1);
    title("ORDERED ODD-EVEN Hopscotch");
    _plegctl = {2 7 7.75 0.75};
    xy(t,sol'-U1');
  setwind(2);
    _plegctl = {2 7 -1.25 0.75};
    title("LINE Hopscotch");
    xy(t,sol'-U2');
    graphprt("-c=1 -cf=ex3.eps");
  endwind;

