/*
**> ex5.prg
**
** Kurpiel, A. and T. Roncalli [1998], Hopscotch methods for two-state
** financial models, FERC working paper, City University Business School
*/

new;
library option,PDE2D;
PDE2Dset;

K = 100;
S0 = seqa(80,5,9);
b = -0.04;
r =  0.08;
V0 = 0.20^2;
tau = 0.25;

rho = -0.5;
Kappa = 0.9;
theta = 0.20^2;
lambda = 0.0;
sigmaV = 0.10;
kappaStar = kappa + lambda;
thetaStar = kappa*theta/(kappa+lambda);

proc aProc(t,x,y);
  retp( 0.5 * y .* (x^2) );
endp;

proc bProc(t,x,y);
  retp( 0.5* rho * sigmaV * y .* x );
endp;

proc cProc(t,x,y);
  retp( 0.5 * (sigmaV^2).*y .* ones(rows(x),1) );
endp;

proc dProc(t,x,y);
  retp( b.*x.*ones(1,cols(y)) );
endp;

proc eProc(t,x,y);
  retp( ( kappaStar*(thetaStar-y) ) .* ones(rows(x),1) );
endp;

proc fProc(t,x,y);
  retp( r .* ones(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

@<------------------ h(t,x,y,U) ------------------>@

proc hProc(t,x,y,U);
  local PayOff,cnd;
  PayOff = ( x .> K ) .* ( x - K ) .* ones(1,cols(y));
  cnd = PayOff .> U;
  retp( cnd.*PayOff + (1-cnd).*U );
endp;

@<------------------------------------------------>@

proc tminBound(t,x,y);
  x = ones(1,cols(y)) .* x;
  retp( (x .> K) .* ( x - K) );
endp;

proc xminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DxmaxBound(t,x,y);
  retp( ones(rows(x),cols(y)) );
endp;

proc DyminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DymaxBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,&hProc,
                  &tminBound,&xminBound,0,0,0,
                  0,&DxmaxBound,0,0);

xmin = 50; xmax = 150; Nx = 201;
ymin = 0.002; ymax = 0.122; Ny = 61;
tmin = 0; tmax = tau; Nt = floor(1825*tau);

_Hopscotch_SaveLastIter = 1;
_Hopscotch_PrintIters = 1;
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");

x = ExtractSolution("tmp","x");
y = ExtractSolution("tmp","y");
tmax = ExtractSolution("tmp","t");
U = ExtractSolution("tmp","t"|tmax);

_fcmptol = 10;
indxX = FindIndex(x,S0); indxY = FindIndex(y',V0);
S0 = x[indxX]; V0 = y[indxY];

Cam = AmericanBS(S0,K,sqrt(V0),tau,b,r);

output file = ex5.out reset;

print S0~Cam~U[indxX,indxY];

output off;



